import click
from .funcs import (
    initialize
)
from ..package import MYPReader
from ..__main__ import main


# Initialize MYP
@main.command("init")
@click.option('-t', '--template', 'template', help="Name of template", prompt='Which template you wanna use?', default='blank')
@click.option('--disable-oninit', 'disable_oninit', is_flag=True, help="Disable work of oninit event")
def init_command(template, disable_oninit, **kwargs):
    """Initialize MYP in current folder"""
    initialize(
        template,
        disable_oninit=disable_oninit
    )


# Set command
@main.command("set")
@click.argument('name', required=True)
@click.option('-v', '--value', 'value', required=False,help="Value to assing to key")
def set_command(name, value):
    """Set value of key in myp.json"""
    myp:MYPReader = MYPReader()
    new_value = value or input(f'Enter new {name} [{myp.get_data(name)}]: ')
    old_value = myp.get_data(name)
    if new_value and new_value != old_value:
        myp.set_data(name, new_value)
        myp.write()
        click.echo(f'Successfully updated {name}: {old_value} -> {new_value}')


# Show command
@main.command("show")
@click.argument('name', required=True)
def get_commmand(name):
    """Show value of key in myp.json"""
    myp:MYPReader = MYPReader()
    click.echo(myp.get_data(name))



flaskmng = {
    "body": {
        "dependencies": {
            "common": [
                'alembic',
                'astroid',
                'bcrypt',
                'blinker',
                'cffi',
                'click',
                'colorama',
                'dnspython',
                'email-validator',
                'Flask',
                'Flask-Bcrypt',
                'Flask-Login',
                'Flask-Mail',
                'Flask-Migrate',
                'Flask-SQLAlchemy',
                'Flask-WTF',
                'idna',
                'isort',
                'itsdangerous',
                'Jinja2',
                'lazy-object-proxy',
                'Mako',
                'MarkupSafe',
                'mccabe',
                'Pillow',
                'pycparser',
                'python-dateutil',
                'python-editor',
                'six',
                'SQLAlchemy',
                'toml',
                'Werkzeug',
                'wrapt',
                'WTForms',
                'wheel'
            ],
            "prod": [
                'uwsgi',
                'gunicorn'
            ],
            "dev": [
                'autopep8',
                'pylint'
            ]
        },
        "scripts": {
            "newapp": {
                "command": "flaskmng startapp",
                "description": "Creating new app for project."
            },
            "rmapp": {
                "command": "flaskmng removeapp",
                "description": "Removing app from project."
            },
            "mkmig": {
                "command": "flask db migrate -m \"{m}\"",
                "description": "Make migrations for project."
            },
            "mig": {
                "command": "flask db upgrade",
                "description": "Upgrade db for project."
            },
            "dbinit": {
                "command": "flask db init",
                "description": "Initialize db for project."
            },
            "start": {
                "command": "flask run",
                "description": "For starting project."
            }
        }
    }
}



import json
from typing import List, Any
from .. import const


class MYPReader:
    data: dict
    file_name: str

    def __init__(self, file_name: str = const.filename):
        with open(file_name) as f:
            data: dict = json.load(f)
        self.data = data
        self.file_name = file_name

    def write(self) -> None:
        """Write object instance to `myp.json`"""
        with open(self.file_name, 'wt') as f:
            json.dump(self.data, f, indent=4, sort_keys=True)

    def get_data(self, key: str) -> Any:
        """Get data from object"""
        return self.data[key]

    def set_data(self, key: str, value: Any) -> None:
        """Set data to object"""
        self.data[key] = value

    def add_script(self, name: str, command: str, description: str) -> str:
        """Add script to object"""
        self.data["scripts"][name] = {
            "command": command,
            "description": description,
        }
        return self.data["scripts"][name]

    def remove_script(self, name: str) -> str:
        """Remove script from object"""
        return self.data["scripts"].pop(name)

    def add_dependency(self, package_name: List[str], scope: str = "common"):
        """Add dependency to object"""
        self.data["dependencies"][scope].extend(package_name)
        self.data["dependencies"][scope] = list(
            set(self.data["dependencies"][scope]))

    def get_dependencies(self, scope: str = "all") -> List[str]:
        """Get dependencies of object"""
        deps: dict = self.data["dependencies"]
        if scope == "all":
            return list(set(deps["dev"]+deps["prod"]+deps["common"]))
        elif scope == "prod":
            return list(set(deps["prod"]+deps["common"]))
        elif scope == "dev":
            return list(set(deps["dev"]+deps["common"]))

        return []



from ..__main__ import main
from ..globals import runScriptDirectly, pip_cmd


@main.command("install")
def install_command():
    """Install dependencies in requirements.txt"""
    runScriptDirectly(pip_cmd(f"install -r requirements.txt"))


@main.command("freeze")
def freeze_command():
    """Output dependencies of project to requirements.txt"""
    runScriptDirectly(pip_cmd(f"freeze > requirements.txt"))
